import com.jscape.inet.mft.management.client.api.ManagerSubsystem;
import com.jscape.inet.vfs.AccessPermissions;
import com.jscape.inet.vfs.Group;
import com.jscape.inet.vfs.VirtualFileDescriptor;
import com.jscape.inet.vfs.VirtualLocalFileDescriptor;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

public class CreateGroupExample {

    private static final BufferedReader CONSOLE_READER = new BufferedReader(new InputStreamReader(System.in));

    private static final String DEFAULT_CLIENT_CONFIG_FILE = "../../../client.cfg";

    private ManagerSubsystem manager;
    private String domainName;
    private String groupName;
    private String realPath;
    private String configFile;

    /**
     * Collects information about the account.
     *
     * @param args command line arguments
     * @throws Exception if an error occurs
     */
    public static void main(String[] args)
            throws Exception {
        showExampleTitle();
        CreateGroupExample example = createExample();
        example.execute();
    }

    /**
     * Shows example title.
     */
    private static void showExampleTitle() {
        System.out.println("This example demonstrates how to add a group using the server API.");
        System.out.println();
    }

    /**
     * Creates an example instance.
     *
     * @return an example instance
     * @throws java.io.IOException if an error occurs
     */
    private static CreateGroupExample createExample()
            throws IOException {
        String domainName = getParameter("Enter domain name: ");
        String groupName = getParameter("Enter group name e.g. Accountants: ");
        String realPath = getParameter("Enter default group directory e.g. C:\\home: ");
        File configFile = getConfigFile();

        return new CreateGroupExample(domainName,
                                      groupName,
                                      realPath,
                                      configFile);
    }

    /**
     * Returns a manager configuration file.
     *
     * @return a manager configuration file
     * @throws java.io.IOException if an I/O error occurs
     */
    private static File getConfigFile()
            throws IOException {
        File configFile = new File(DEFAULT_CLIENT_CONFIG_FILE);

        while (!configFile.exists()) {
            System.out.println("Configuration file not found.");
            String configFilePath = getParameter("Please enter absolute path of client.cfg file: ");
            configFile = new File(configFilePath);
        }

        return configFile;
    }

    /**
     * Support method. Shows prompt on console and reads user response.
     *
     * @param prompt prompt string
     * @return user response
     * @throws java.io.IOException if an I/O error occurs
     */
    private static String getParameter(String prompt)
            throws IOException {
        System.out.print(prompt);
        return CONSOLE_READER.readLine();
    }

    /**
     * Constructs example.
     *
     * @param domainName domain name
     * @param groupName  group name
     * @param realPath   user login directory
     * @param configFile config file
     */
    public CreateGroupExample(String domainName, String groupName, String realPath, File configFile) {
        this.domainName = domainName;
        this.groupName = groupName;
        this.realPath = realPath;
        this.configFile = configFile.getAbsolutePath();
    }

    /**
     * Creates group and adds to server.
     */
    public void execute() {
        try {
            // create manager
            this.manager = new ManagerSubsystem(this.configFile);

            // establish connection
            this.manager.connect();

            // create group
            Group group = createGroup();

            // add account
            this.manager.addGroup(this.domainName, group);

            System.out.println("Group created successfully!");
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // disconnect
            this.manager.disconnect();
        }
    }

    /**
     * Creates a group.
     *
     * @return a group
     */
    private Group createGroup() {
        VirtualFileDescriptor path = createVirtualFileDescriptor();
        VirtualFileDescriptor[] descriptorList = new VirtualFileDescriptor[]{path};

        return new Group(this.groupName, descriptorList);
    }

    /**
     * Creates virtual file descriptor.
     *
     * @return the virtual file descriptor
     */
    private VirtualFileDescriptor createVirtualFileDescriptor() {
        return new VirtualLocalFileDescriptor("/", new AccessPermissions(), this.realPath, true);
    }

}
