import com.jscape.inet.mft.management.client.api.ManagerSubsystem;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

public class StartDomainExample {

    private static final BufferedReader CONSOLE_READER = new BufferedReader(new InputStreamReader(System.in));

    private static final String DEFAULT_CLIENT_CONFIG_FILE = "../../../client.cfg";

    private ManagerSubsystem manager;
    private String domainName;
    private String configFile;

    /**
     * Collects information about the account.
     *
     * @param args command line arguments
     * @throws Exception if an error occurs
     */
    public static void main(String[] args)
            throws Exception {
        showExampleTitle();
        StartDomainExample example = createExample();
        example.execute();
    }

    /**
     * Shows example title.
     */
    private static void showExampleTitle() {
        System.out.println("This example demonstrates how to start a domain using the server API.");
        System.out.println();
    }

    /**
     * Creates an example instance.
     *
     * @return an example instance
     * @throws java.io.IOException if an error occurs
     */
    private static StartDomainExample createExample()
            throws IOException {
        String domainName = getParameter("Enter domain name: ");
        File configFile = getConfigFile();

        return new StartDomainExample(domainName, configFile);
    }

    /**
     * Returns a manager configuration file.
     *
     * @return a manager configuration file
     * @throws java.io.IOException if an I/O error occurs
     */
    private static File getConfigFile()
            throws IOException {
        File configFile = new File(DEFAULT_CLIENT_CONFIG_FILE);

        while (!configFile.exists()) {
            System.out.println("Configuration file not found.");
            String configFilePath = getParameter("Please enter absolute path of client.cfg file: ");
            configFile = new File(configFilePath);
        }

        return configFile;
    }

    /**
     * Support method. Shows prompt on console and reads user response.
     *
     * @param prompt prompt string
     * @return user response
     * @throws java.io.IOException if an I/O error occurs
     */
    private static String getParameter(String prompt)
            throws IOException {
        System.out.print(prompt);
        return CONSOLE_READER.readLine();
    }

    /**
     * Constructs example.
     *
     * @param domainName domain name
     * @param configFile config file
     */
    public StartDomainExample(String domainName, File configFile) {
        this.domainName = domainName;
        this.configFile = configFile.getAbsolutePath();
    }

    /**
     * Starts domain.
     */
    public void execute() {
        try {
            // create manager
            this.manager = new ManagerSubsystem(this.configFile);

            // establish connection
            this.manager.connect();

            // start domain
            this.manager.startDomain(this.domainName);

            System.out.println("Domain started!");
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // disconnect
            this.manager.disconnect();
        }
    }

}
