<%@ tag body-content="scriptless" dynamic-attributes="args" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/fmt" prefix="fmt" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>
<%@ taglib tagdir="/WEB-INF/tags" prefix="tags" %>
<%@ attribute name="type" description="The button type [button, submit, reset]. Default type - submit." %>
<%@ attribute name="label" description="The label that will appear on the element. If this is left out, no text appears." %>
<%@ attribute name="labelkey" description="The label key in resource bundle. If this is left out, no text appears." %>
<%@ attribute name="name" description="Used to specify the button name as part of the name/value pair sent to the server." %>
<%@ attribute name="icon" description="The URL of the icon to appear on the element. The position of the icon is determined by the orient attribute." %>
<%@ attribute name="orient" description="Used to specify whether the label of button is oriented HORIZONTAL or VERTICAL. The default value - HORIZONTAL" %>
<%@ attribute name="shortcut" description="The button shortcut key." %>
<%@ attribute name="disabled" type="java.lang.Boolean" %>
<%@ attribute name="disabledClass" %>

<c:set var="buttonType">
    <c:choose>
        <c:when test="${type eq 'button' or type eq 'reset'}">${type}</c:when>
        <c:otherwise>submit</c:otherwise>
    </c:choose>
</c:set>

<c:set var="buttonText">
    <c:choose>
        <c:when test="${not empty labelkey}"><fmt:message key="${labelkey}"/></c:when>
        <c:otherwise><c:out value="${label}"/></c:otherwise>
    </c:choose>
    <c:if test="${not empty shortcut}"> - ${fn:escapeXml(shortcut)}</c:if>
</c:set>
<c:set var="buttonLabel" value="${fn:replace(buttonText, ' ', '&nbsp;')}" />
<c:set var="buttonTitle">${buttonText}</c:set>

<c:set var="buttonIcon">
    <c:if test="${not empty icon}">
        <c:url var="buttonIconUrl" value="${icon}" />
        <c:set var="buttonIconDisabled">
            <c:if test="${disabled and not empty disabledClass}"> class="<c:out value="${disabledClass}" />"</c:if>
        </c:set>
        <img src="${buttonIconUrl}" alt="${buttonTitle}" ${buttonIconDisabled} />
    </c:if>
</c:set>


<%-- button attributes --%>
<c:if test="${not empty name}">
    <c:set target="${args}" property="name" value="${name}"/>
</c:if>
<c:if test="${disabled}">
    <c:set target="${args}" property="disabled" value="disabled"/>
</c:if>
<c:if test="${not empty buttonTitle}">
    <c:set target="${args}" property="title" value="${buttonTitle}"/>
</c:if>
<tags:tagDynamicAttributes dynamicAttributes="${args}" />


<c:set var="buttonText">
    <c:if test="${not empty buttonLabel}">
        <c:choose>
            <c:when test="${fn:toUpperCase(orient) eq 'VERTICAL'}"><div>${buttonLabel}</div></c:when>
            <c:when test="${not empty buttonIcon}"><span>${buttonLabel}</span></c:when>
            <c:otherwise>${buttonLabel}</c:otherwise>
        </c:choose>
    </c:if>
</c:set>

<button type="${buttonType}" ${tagAttributes}>${buttonIcon}${buttonText}<jsp:doBody/></button>