function UploadService(url) {
    this._constructor(url);
}
(function() { // create local scope;

    var IFRAME_NAME = "uploadIFrame";
    var IFRAME_ID = "uploadIFrame";
    var IFRAME_START_URL = "/empty.html";
    var BUTTONS_BAR_ID = "buttonsbar";
    var MESSAGE_BAR_ID = "messagebar";
    var LIST_TABLE_ID = "list";
    var JS_HOLDER_ID = "jsPreparePollHolder";
    var MAX_JS_HOLDERS = 10;
    var POLLING_TIMEOUT = 1000;
    var DEFAULT_TIMEOUT = 60000;
    var DEFAULT_DECAY = 2;

    UploadService.prototype = {

        url: null,
        iframe: null,
        jsHolders: null,
        jsHolderIndex: null,
        progressBar: null,
        decay: null,
        currentDecay: null,
        timeout: null,
        timeoutTimer: null,
        pollTimer: null,
        lastText: null,
        lastActivityTime: null,
        startTime: null,

        _constructor: function(url) {
            this.url = url;
            this.iframe = this._createIFrame();
            this.jsHolders = new Array();
            this.jsHolderIndex = 0;
            this.progressBar = new ProgressBar();
            this.setTimeout(DEFAULT_TIMEOUT);
            this.setDecay(DEFAULT_DECAY);
        },

        setTimeout: function(timeout) {
            this.timeout = timeout || DEFAULT_TIMEOUT;
        },

        setDecay: function(decay) {
            this.decay = decay || DEFAULT_DECAY;
            this.currentDecay = this.decay;
        },

        start: function(form) {
            if (form == null) return;

            form.target = IFRAME_NAME;
            this._submitForm(form);

            var messageBar = document.getElementById(MESSAGE_BAR_ID);
            var buttonsBar = document.getElementById(BUTTONS_BAR_ID);
            var listPanel = document.getElementById(LIST_TABLE_ID);

            Utils.setVisible(messageBar, false);
            Utils.setVisible(buttonsBar, false);
            Utils.setVisible(listPanel, false);
            this.progressBar.setVisible(true);
        },

        progress: function(percent, statusMessage, timeMessage) {
            this.progressBar.setProgress(percent);
            this.progressBar.setStatusInfo(statusMessage);
            this.progressBar.setTimeInfo(timeMessage);
            this.lastActivityTime = new Date().getTime();

            window.clearTimeout(this.timeoutTimer);
            var thisObj = this;
            this.timeoutTimer = window.setTimeout(function() {
                thisObj._checkTimeout();
            }, this.timeout);
        },

        prepare: function(percent, statusMessage, timeMessage) {
            this.progress(percent, statusMessage, timeMessage);
            this._onPollComplete(statusMessage + timeMessage);
        },

        prepareComplete: function() {
            this._clearTimer();
        },

        finish: function() {
            this._clearTimer();
        },

        cancel: function() {
            this._clearTimer();
            this.iframe.src = IFRAME_START_URL;
        },

        _submitForm: function(form) {
            this.lastActivityTime = new Date().getTime();

            if (Utils.isSafariNavigator()) {
                setTimeout(function() {
                    form.submit();
                }, 50);
            } else {
                form.submit();
            }

            var thisObj = this;
            this.pollTimer = window.setTimeout(function() {
                thisObj._poll();
            }, POLLING_TIMEOUT);
        },

        _poll: function() {
            // add time to url to prevent caching
            var url = this.url + "?t=" + new Date().getTime();
            var jsHolder = this._getJSHolder();

            setTimeout(function() {
                var js = jsHolder.getElementsByTagName("script")[0];
                js.setAttribute("type", "text/javascript");
                js.setAttribute("language", "javascript");
                js.setAttribute("src", encodeURI(url));
            }, 10);

            this.jsHolder = jsHolder;

            var thisObj = this;
            this.timeoutTimer = window.setTimeout(function() {
                thisObj._checkTimeout();
            }, this.timeout);
        },

        _onPollComplete: function(text) {
            this.lastActivityTime = new Date().getTime();
            this.currentDecay = this.lastText == text ? this.currentDecay + this.decay : 1;
            this.lastText = text;

            var thisObj = this;
            this.pollTimer = window.setTimeout(function() {
                thisObj._poll();
            }, POLLING_TIMEOUT * this.currentDecay);
        },

        _checkTimeout: function() {
            if ((new Date().getTime() - this.lastActivityTime) > this.timeout) {
                this.cancel();
                window.onConnectionTimeout();
            }
        },

        _clearTimer: function() {
            window.clearTimeout(this.pollTimer);
            window.clearTimeout(this.timeoutTimer);
        },

        _getJSHolder: function() {
            if (++this.jsHolderIndex >= MAX_JS_HOLDERS) {
                this.jsHolderIndex = 0;
            }

            this._clearJSHolder(this.jsHolderIndex);

            var id = JS_HOLDER_ID + this.jsHolderIndex;
            var jsHolder = this._createJSHolder(id);
            this.jsHolders[this.jsHolderIndex] = jsHolder;

            return jsHolder;
        },

        _createJSHolder: function(id) {
            var result = document.body.appendChild(document.createElement("span"));
            result.setAttribute("id", id);
            result.style.display = "none";
            result.innerHTML = "Text for IE.<s" + "cript></" + "script>";

            return result;
        },

        _clearJSHolder: function(index) {
            if (this.jsHolders[index] != null) {
                var id = this.jsHolders[index].getAttribute("id");
                this.jsHolders[index] = null;

                var holder = document.getElementById(id);

                setTimeout(function() { // without setTimeout() - crash in IE 5.0!
                    if (holder != null) {
                        holder.parentNode.removeChild(holder);
                    }
                }, 50);
            }
        },

        _createIFrame: function() {
            var result = document.getElementById(IFRAME_ID);

            if (result == null) {
                var holder = document.body.appendChild(document.createElement("div"));
                holder.innerHTML = "<iframe src='" + IFRAME_START_URL + "' "
                        + "id='" + IFRAME_ID + "' "
                        + "name='" + IFRAME_NAME + "' "
                        + "style='width: 0; height: 0; border: none;'></iframe>";
                holder.style.position = "relative";
                holder.style.top = "0px";
                holder.style.left = "0px";

                result = holder.getElementsByTagName("iframe")[0];
            }

            return result;
        }

    }

})();