function $(id) {
    return document.getElementById(id);
}
function $F(elem) {
    return Utils.isSelect(elem)
            ? Utils.getSelectedValue(elem)
            : ((Utils.isRadioInput(elem) || Utils.isCheckboxInput(elem))
                ? (elem.checked ? elem.value : null)
                : (Utils.isInputTag(elem, Tag.INPUT_TYPE_TEXT) ? elem.value : null));
}
Array.prototype.first = function() {
    return this[0];
};
Array.prototype.last = function() {
    return this[this.length - 1];
};
if (!Array.prototype.indexOf) {
    Array.prototype.indexOf = function(needle, offset) {
        for (var i = offset || 0, size = this.length; i < size; ++i) {
            if (this[i] === needle) {
                return i;
            }
        }
        return -1;
    };
}
String.prototype.trim = function () {
    return this.replace(/^\s+|\s+$/g, "");
};

document.viewport = {
    getDimensions: function() {
        return Utils.isSafariNavigator() && !document.evaluate
                ? {width: window.innerWidth, height: window.innerHeight}
                : Utils.isOpera()
                ? {width: document.body.clientWidth, height: document.body.clientHeight}
                : {width: document.documentElement.clientWidth, height: document.documentElement.clientHeight}
    },

    getScrollOffsets: function() {
        var top = window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop;
        var left = window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft;
        return {top: top, left: left};
    },

    getOffsetFor: function(elem) {
        var result = {top: 0, left: 0};

        for (var el = elem; el != null; el = el.offsetParent) {
            result.top += el.offsetTop || 0;
            result.left = el.offsetLeft || 0;
            // Safari bug
            if (el.offsetParent == document.body && el.style.position == 'absolute') {
                break;
            }
        }

        for (var el = elem; el != null; el = el.parentNode) {
            if (!Utils.isOpera() || el.tagName == Tag.BODY) {
                result.top -= el.scrollTop || 0;
                result.left -= el.scrollLeft || 0;
            }
        }

        return result;
    }
};

function Utils() {
    this._constructor();
}(function() { // create local scope;

    Utils.isGeckoNavigator = function() {
        return (navigator.product == "Gecko");
    };

    Utils.isSafariNavigator = function() {
        return (navigator.userAgent.indexOf("Safari") != -1);
    };

    Utils.isIEMacNavigator = function() {
        return (navigator.userAgent.indexOf("Mac") != -1) && (navigator.userAgent.indexOf("MSIE") != -1);
    };

    Utils.isOpera = function() {
        return !!window.opera;
    };

    Utils.getInnerText = function(elem) {
        if (typeof elem == Type.STRING || typeof elem == Type.UNDEFINED) {
            return elem;
        }

        if (elem.nodeType == Node.TEXT) {
            return Utils.getInnerText(elem.parentNode);
        }

        if (elem.innerText) {
            return elem.innerText;
        }

        var result = "";

        for (var el = elem.firstChild; el != null; el = el.nextSibling) {
            switch (el.nodeType) {
                case Node.ELEMENT:
                    result += Utils.getInnerText(el);
                    break;

                case Node.TEXT:
                    result += el.nodeValue;
                    break;
            }
        }

        return result;
    };

    Utils.getTextNode = function(elem) {
        if (typeof elem == Type.UNDEFINED) {
            return null;
        }

        for (var el = elem.firstChild; el != null; el = el.nextSibling) {
            if (el.nodeType == Node.TEXT) {
                return el;
            }
        }

        return null;
    };

    Utils.compareIgnoreCase = function(str1, str2) {
        if (str1 != null && str2 != null) {
            var s1 = str1.toString().toLowerCase();
            var s2 = str2.toString().toLowerCase();

            return (s1 == s2 ? 0 : (s1 < s2 ? -1 : 1));
        }

        return 0;
    };

    Utils.equalsIgnoreCase = function(str1, str2) {
        return (Utils.compareIgnoreCase(str1, str2) == 0);
    };

    Utils.createLabel = function(title, htmlFor) {
        var result = document.createElement(Tag.LABEL);
        result.appendChild(document.createTextNode(title));
        result.htmlFor = (typeof htmlFor == Type.STRING) ? htmlFor : "";

        return result;
    }

    Utils.createFileInput = function(name) {
        return Utils.createInput(Tag.INPUT_TYPE_FILE, name);
    };

    Utils.createRadioInput = function(name) {
        return Utils.createInput(Tag.INPUT_TYPE_RADIO, name);
    };

    Utils.createTextInput = function(name) {
        return Utils.createInput(Tag.INPUT_TYPE_TEXT, name);
    };

    Utils.createInput = function(type, name) {
        var result = null;

        if (document.all && document.createElement && !Utils.isOpera()) {
            result = document.createElement("<" + Tag.INPUT + " type=\"" + type + "\" name=\"" + name + "\" />");
        } else if (document.createElement) {
            result = document.createElement(Tag.INPUT);
            result.type = type;
            result.name = name;
        }

        return result;
    };

    Utils.isTag = function(elem, tagName) {
        return (elem != null && elem.nodeType == Node.ELEMENT && elem.tagName == tagName);
    };

    Utils.isInputTag = function(elem, type) {
        return (Utils.isTag(elem, Tag.INPUT) && elem.type == type);
    };

    Utils.isCheckboxInput = function(elem) {
        return Utils.isInputTag(elem, Tag.INPUT_TYPE_CHECKBOX);
    };

    Utils.isRadioInput = function(elem) {
        return Utils.isInputTag(elem, Tag.INPUT_TYPE_RADIO);
    };

    Utils.isFileInput = function(elem) {
        return Utils.isInputTag(elem, Tag.INPUT_TYPE_FILE);
    };

    Utils.getFormInputs = function(form, type, name) {
        var inputs = form.getElementsByTagName(Tag.INPUT);
        if (!type && !name) { return inputs; }

        var result = new Array();
        for (var i = 0; i < inputs.length; ++i) {
            var victim = inputs[i];
            if ((type && victim.type != type) || (name && victim.name != name)) {
                continue;
            }
            result.push(victim);
        }
        return result;
    };

    Utils.isSelect = function(elem) {
        return Utils.isTag(elem, Tag.SELECT);
    };

    Utils.clearSelect = function(elem) {
        if (Utils.isSelect(elem)) {
            elem.options.length = 0;
        }
    };

    Utils.getSelectedValue = function(select) {
        return (Utils.isSelect(select) && select.selectedIndex != -1)
                ? select.options[select.selectedIndex].value
                : null;
    };

    Utils.isBody = function(elem) {
        return Utils.isTag(elem, Tag.BODY);
    };

    Utils.isLink = function(elem) {
        return Utils.isTag(elem, Tag.LINK);
    };

    Utils.isCell = function(elem) {
        return Utils.isTag(elem, Tag.TABLE_CELL);
    };

    Utils.isHeader = function(elem) {
        return Utils.isTag(elem, Tag.TABLE_HEAD_CELL)
                || (Utils.isCell(elem) && Utils.isTag(elem.parentNode.parentNode, Tag.TABLE_HEAD));
    };

    Utils.isRow = function(elem) {
        return Utils.isTag(elem, Tag.TABLE_ROW);
    };

    Utils.isHeaderRow = function(elem) {
        if (!Utils.isRow(elem) || elem.cells.length == 0) {
            return false;
        }

        for (var i = 0; i < elem.cells.length; ++i) {
            if (!Utils.isHeader(elem.cells[i])) {
                return false;
            }
        }

        return true;
    };

    Utils.setEnabled = function(elem, enabled, disabledCssClass) {
        if (elem != null) {
            if (Utils.isTag(elem, Tag.INPUT) || Utils.isSelect(elem) || Utils.isTag(elem, Tag.BUTTON)) {
                elem.disabled = !enabled;
                elem.blur();
            }

            if (disabledCssClass) {
                if (enabled) {
                    Utils.removeCssClass(elem, disabledCssClass)
                } else {
                    Utils.addCssClass(elem, disabledCssClass);
                }
            }

            for (var el = elem.firstChild; el != null; el = el.nextSibling) {
                Utils.setEnabled(el, enabled, disabledCssClass);
            }
        }
    };

    Utils.isVisible = function(elem) {
        return (elem != null) ? elem.style.display != "none" : false;
    };

    Utils.setVisible = function(elem, visible) {
        if (elem == null) {
            return;
        }

        if (Utils.isSafariNavigator()) {
            // todo: hide when found children file input
            if (!elem.originalWidth) {
                elem.originalWidth = elem.style.width ? elem.style.width : "auto";
            }

            elem.style.width = visible ? elem.originalWidth : "0";
            elem.style.position = visible ? "static" : "absolute";
            elem.style.left = visible ? "auto" : "-10000px";
        } else {
            elem.style.display = visible ? "" : "none";
        }
    };

    Utils.setOpacity = function(elem, opacity) {
        var value = (opacity < 0.00001) ? 0 : Math.min(Math.max(opacity, 0), 1);
        var cssValue = (value == 1 ? '' : value);
        var currentStyle = elem.currentStyle;
        var css = elem.style;

        if ((currentStyle && !currentStyle.hasLayout) || (!currentStyle && elem.style.zoom == 'normal')) {
            elem.style.zoom = 1;
        }

        if (typeof css.filter != Type.UNDEFINED) {
            css.filter = (value < 1) ? "alpha(opacity=" + value * 100 + ")" : "";
        } else if (typeof css.MozOpacity != Type.UNDEFINED) {
            css.MozOpacity = cssValue;
        } else if (typeof css.KHTMLOpacity != Type.UNDEFINED) {
            css.KHTMLOpacity = cssValue;
        } else if (typeof css.KhtmlOpacity != Type.UNDEFINED) {
            css.KhtmlOpacity = cssValue;
        } else if (typeof css.opacity != Type.UNDEFINED) {
            css.opacity = cssValue;
        }
    };

    Utils.setStyle = function(elem, styles) {
        if (typeof styles == Type.STRING) {
            elem.style.cssText += ";" + styles;
            var opacityStyleReg = /opacity:\s*(\d?\.?\d*)/;
            if (opacityStyleReg.test(styles)) {
                Utils.setOpacity(elem, RegExp.$1);
            }
        } else {
            var css = elem.style;
            for (var property in styles) {
                if (property == 'opacity') {
                    Utils.setOpacity(elem, styles[property]);
                } else if (property == 'float' || property == 'cssFloat') {
                    var key = (typeof css.styleFloat == Type.UNDEFINED) ? 'cssFloat' : 'styleFloat';
                    css[key] = styles[property];
                } else {
                    css[property] = styles[property];
                }
            }
        }
    };

    Utils.getDimension = function(elem) {
        var width = 0;
        var height = 0;

        if (Utils.isVisible(elem)) {
            width = elem.offsetWidth;
            height = elem.offsetHeight;
        } else {
            var origVisibility = elem.style.visibility;
            var origPosition = elem.style.position;
            var origDisplay = elem.style.display;
            elem.style.visibility = 'hidden';
            elem.style.position = 'absolute';
            elem.style.display = 'block';

            width = elem.offsetWidth;
            height = elem.offsetHeight;

            elem.style.display = origDisplay;
            elem.style.position = origPosition;
            elem.style.visibility = origVisibility;
        }

        return {width: width, height: height};
    };

    Utils.addCssClass = function(elem, cssClass) {
        var classNames = Utils._getCssClasses(elem);

        if (!Utils.containsCssClass(elem, cssClass)) {
            classNames[classNames.length] = cssClass;
            Utils._setCssClasses(elem, classNames);
        }
    };

    Utils.removeCssClass = function(elem, cssClass) {
        var classNames = Utils._getCssClasses(elem);

        for (var i = 0; i < classNames.length; ++i) {
            if (classNames[i] == cssClass) {
                delete classNames[i];
            }
        }

        Utils._setCssClasses(elem, classNames);
    };

    Utils.containsCssClass = function(elem, cssClass) {
        var classNames = Utils._getCssClasses(elem);

        for (var i = 0; i < classNames.length; ++i) {
            if (classNames[i] == cssClass) {
                return true;
            }
        }

        return false;
    };

    Utils._getCssClasses = function(elem) {
        if (elem != null && elem.nodeType == Node.ELEMENT) {
            var elemClassName = elem.className.trim();
            if (elemClassName.length != 0) {
                return elemClassName.split(/\s+/);
            }
        }

        return  new Array();
    };

    Utils._setCssClasses = function(elem, cssClasses) {
        if (elem != null && elem.nodeType == Node.ELEMENT) {
            //todo: check performance in IE
            elem.className = cssClasses.join(" ");
        }
    };

    Utils.prototype = {
        _constructor: function() {
        }
    }

})();


function Type() {
    // create local scope;
}
(function() {

    Type.BOOLEAN = "boolean";
    Type.FUNCTION = "function";
    Type.NUMBER = "number";
    Type.OBJECT = "object";
    Type.STRING = "string";
    Type.UNDEFINED = "undefined";

})();


function Tag() {
    // create local scope;
}
(function() {

    Tag.BODY = "BODY";
    Tag.DIV = "DIV";
    Tag.SPAN = "SPAN";
    Tag.LINK = "A";
    Tag.TABLE = "TABLE";
    Tag.TABLE_CAPTION = "CAPTION";
    Tag.TABLE_HEAD = "THEAD";
    Tag.TABLE_BODY = "TBODY";
    Tag.TABLE_FOOT = "TFOOT";
    Tag.TABLE_ROW = "TR";
    Tag.TABLE_CELL = "TD";
    Tag.TABLE_HEAD_CELL = "TH";
    Tag.LABEL = "LABEL";
    Tag.SELECT = "SELECT";
    Tag.BUTTON = "BUTTON";
    Tag.IMAGE = "IMG";
    Tag.INPUT = "INPUT";
    Tag.INPUT_TYPE_HIDDEN = "hidden";
    Tag.INPUT_TYPE_BUTTON = "button";
    Tag.INPUT_TYPE_CHECKBOX = "checkbox";
    Tag.INPUT_TYPE_RADIO = "radio";
    Tag.INPUT_TYPE_FILE = "file";
    Tag.INPUT_TYPE_TEXT = "text";
    Tag.OPTION = "option";

})();


function Node() {
    // create local scope;
}
(function() {

    Node.ELEMENT = 1;
    Node.ATTRIBUTE = 2;
    Node.TEXT = 3;
    Node.CDATA_SECTION = 4;
    Node.ENTITY_REFERENCE = 5;
    Node.ENTITY = 6;
    Node.PROCESSING_INSTRUCTION = 7;
    Node.COMMENT = 8;
    Node.DOCUMENT = 9;
    Node.DOCUMENT_TYPE = 10;
    Node.DOCUMENT_FRAGMENT = 11;
    Node.NOTATION = 12;

})();