function NumberEditor(control) {
    this._constructor(control);
}(function() { // create local scope;

    NumberEditor.ALLOWED_KEY_CODES = [
        KeyEvent.VK_ESCAPE,
        KeyEvent.VK_LEFT,
        KeyEvent.VK_RIGHT,
        KeyEvent.VK_UP,
        KeyEvent.VK_DOWN,
        KeyEvent.VK_PAGE_UP,
        KeyEvent.VK_PAGE_DOWN,
        KeyEvent.VK_BACK_SPACE,
        KeyEvent.VK_DELETE,
        KeyEvent.VK_TAB
    ];

    NumberEditor.prototype = {

        control: null,
        minValue: null,
        maxValue: null,
        allowedChars: null,

        _constructor: function(control) {
            this.control = control;
            this.allowedChars = [''];
            this.minValue = Number.MIN_VALUE;
            this.maxValue = Number.MAX_VALUE;

            var thisObj = this;
            this.control.onkeypress = function(event) { return thisObj._handle(event); }
        },

        addAllowedChar: function(c) {
            this.allowedChars.push(c);
        },

        setMinValue: function(value) {
            this.minValue = parseInt(value, 10);
        },

        setMaxValue: function(value) {
            this.maxValue = parseInt(value, 10);
        },

        getValue: function() {
            var result = parseInt(this.control.value, 10);
            return isNaN(result) ? 0 : result;
        },

        setValue: function(value) {
            this.control.value = Math.max(this.minValue, Math.min(this.maxValue, parseInt(value, 10)));
        },

        toString: function() {
            return this.control.value;
        },

        _handle: function(event) {
            var ke = new KeyEvent(event);
            var code = ke.getKeyCode();

            if (ke.isModifiersDown()
                || !(this._isAllowedKey(code) || this._isAllowedChar(ke.getChar()) || this._isDigit(code))) {
                return ke.cancel();
            }

            if (code == KeyEvent.VK_UP || code == KeyEvent.VK_PAGE_UP) {
                this.setValue(this.getValue() + 1);
                return ke.cancel();
            } else if (code == KeyEvent.VK_DOWN || code == KeyEvent.VK_PAGE_DOWN) {
                this.setValue(this.getValue() - 1);
                return ke.cancel();
            } else if (code == KeyEvent.VK_ESCAPE) {
                this.control.value = this.control.defaultValue;
                return ke.cancel();
            }

            return true;
        },

        _isDigit: function(code) {
            return code >= 48 && code <= 57;
        },

        _isAllowedChar: function(c) {
            for (var i = 0; i < this.allowedChars.length; ++i) {
                if (c == this.allowedChars[i]) {
                    return true;
                }
            }

            return false;
        },

        _isAllowedKey: function(code) {
            for (var i = 0; i < NumberEditor.ALLOWED_KEY_CODES.length; ++i) {
                if (code === NumberEditor.ALLOWED_KEY_CODES[i]) {
                    return true;
                }
            }

            return false;
        }

    }

})();