function ManagePathsPane(pathBoxs, selectAllBox, upButton, downButton, editButton, deleteButton) {
    this._constructor(pathBoxs, selectAllBox, upButton, downButton, editButton, deleteButton);
}(function() { // create local scope;

    ManagePathsPane.DELETE_CONFIRM_MESSAGE = "deleteConfirmMsg";

    ManagePathsPane.prototype = {

        pathBoxs: null,
        selectAllBox: null,
        upButton: null,
        downButton: null,
        editButton: null,
        deleteButton: null,
        messages: null,

        _constructor: function(pathBoxs, selectAllBox, upButton, downButton, editButton, deleteButton) {
            this.pathBoxs = pathBoxs;
            this.selectAllBox = selectAllBox;
            this.upButton = upButton;
            this.downButton = downButton;
            this.editButton = editButton;
            this.deleteButton = deleteButton;
            this.messages = [];

            this._attachEvent();
            this._update();
        },

        addMessage: function(type, message) {
            this.messages.push({type: type, message: message});
        },

        getMessage: function(type) {
            for (var i = 0; i < this.messages.length; ++i) {
                var item = this.messages[i];
                if (item.type == type) {
                    return item.message;
                }
            }

            return null;
        },

        getPaths: function() {
            var result = new Array();
            for (var i = 0, size = this.pathBoxs.length; i < size; ++i) {
                result.push(this.pathBoxs[i].value);
            }
            return result;
        },

        getSelectedCount: function() {
            var count = 0;
            for (var i = 0, size = this.pathBoxs.length; i < size; ++i) {
                if (this.pathBoxs[i].checked) { count++; }
            }
            
            return count;
        },

        getSelectedIndex: function() {
            for (var i = 0, size = this.pathBoxs.length; i < size; ++i) {
                if (this.pathBoxs[i].checked) {
                    return i;
                }
            }

            return -1;
        },

        selectAll: function(value) {
            for (var i = 0, size = this.pathBoxs.length; i < size; ++i) {
                this.pathBoxs[i].checked = value;
            }
            this._update();
        },

        handleMoveUp: function() {
            if (this.getSelectedCount() == 1) {
                var index = this.getSelectedIndex();
                if (index != 0) {
                    this._swapPaths(this.pathBoxs, index - 1, index);
                    this._update();
                }
            }
        },

        handleMoveDown: function() {
            if (this.getSelectedCount() == 1) {
                var index = this.getSelectedIndex();
                if (index != this.pathBoxs.length - 1) {
                    this._swapPaths(this.pathBoxs, index, index + 1);
                    this._update();
                }
            }
        },

        handleEdit: function() {
            return this.getSelectedCount() != 0;
        },

        handleDelete: function() {
            if (this.getSelectedCount() != 0 && this.pathBoxs.length > 1) {
                var message = this.getMessage(ManagePathsPane.DELETE_CONFIRM_MESSAGE);
                var confirmDelete = (message != null) ? confirm(message) : true;
                if (confirmDelete) {
                    var newPaths = [];
                    for (var i = 0, size = this.pathBoxs.length; i < size; ++i) {
                        var box = this.pathBoxs[i];
                        if (box.checked) {
                            var row = this._getParentRow(box);
                            row.parentNode.removeChild(row);
                        } else {
                            newPaths.push(box);
                        }
                    }
                    this.pathBoxs = newPaths;
                    this._update();
                    return true;
                }
            }
            return false;
        },

        handleSelectAll: function() {
            this.selectAll(this.selectAllBox.checked);
        },

        _swapPaths: function(pathBoxs, index1, index2) {
            var tmp = pathBoxs[index1];
            pathBoxs[index1] = pathBoxs[index2];
            pathBoxs[index2] = tmp;

            var row1 = this._getParentRow(pathBoxs[index1]);
            var row2 = this._getParentRow(pathBoxs[index2]);
            row1.parentNode.insertBefore(row1, row2);
        },

        _getParentRow: function(elem) {
            for (var el = elem.parentNode; el != null; el = el.parentNode) {
                if (Utils.isRow(el)) {
                    return el;
                }
            }
            return null;
        },

        _attachEvent: function() {
            var thisObj = this;
            for (var i = 0; i < this.pathBoxs.length; ++i) {
                this.pathBoxs[i].onclick = function() { thisObj._update(); }
            }

            this.selectAllBox.onclick = function() { thisObj.handleSelectAll(); }
            this.upButton.onclick = function() { thisObj.handleMoveUp(); }
            this.downButton.onclick = function() { thisObj.handleMoveDown(); }
            this.editButton.onclick = function() { thisObj.handleEdit(); }
            this.deleteButton.onclick = function() { return thisObj.handleDelete(); }
        },

        _update: function() {
            var selectedCount = this.getSelectedCount();
            var pathCount = this.pathBoxs.length;
            var canMove = (selectedCount == 1 && pathCount > 1);

            this.selectAllBox.checked = (pathCount == selectedCount && pathCount != 0);
            Utils.setEnabled(this.selectAllBox, pathCount != 0);
            Utils.setEnabled(this.upButton, canMove && this.getSelectedIndex() != 0);
            Utils.setEnabled(this.downButton, canMove && this.getSelectedIndex() != pathCount - 1);
            Utils.setEnabled(this.editButton, selectedCount == 1);
            Utils.setEnabled(this.deleteButton, selectedCount != 0 && this.pathBoxs.length > 1);
        }
    }
})();