function ModalDialog(id) {
    this._constructor(id);
}(function() { // create local scope;

    ModalDialog.BUTTON_OK = "OkButton";
    ModalDialog.BUTTON_CANCEL = "CancelButton";
    ModalDialog.BUTTON_CLOSE = "CloseButton";
    ModalDialog._VISIBLE_DIALOGS = [];

    var COVER_ID = "modalCover";
    var TITLE_CSS_CLASS = "mdTitlePane";
    var MODAL_DIALOG_CSS_CLASS = "modalDialog";
    var CONTENT_PANE_CSS_CLASS = "mdContentPane";
    var BUTTONS_BAR_CSS_CLASS = "mdButtonsBar";
    var Z_INDEX = 0x7FFFFE00;
    var DEFAULT_OPACITY = 0.75;
    var TABBED_TAGS = ["A", "AREA", "BUTTON", "INPUT", "OBJECT", "SELECT", "TEXTAREA"];

    ModalDialog.prototype = {

        id: null,
        shim: null,
        cover: null,
        dialog: null,
        titlePane: null,
        contentPane: null,
        buttonsBar: null,
        isIENavigator: null,

        _constructor: function(id) {
            this.isIENavigator = /MSIE\s*(5\.5|6\.)/.test(navigator.userAgent);
            this.id = id || ("ModalDialog" + new Date().getTime());
            this.shim = this._createShim();
            this.cover = this._createCover();

            this._compose();

            this.setTitle(null);
            this.setOpacity(DEFAULT_OPACITY);
            this.setZIndex(Z_INDEX);

            var thisObj = this;
            window.onresize = function() { thisObj.update(); }
            this.update();
        },

        addButton: function(type, text, handler) {
            var button = Utils.createInput(Tag.INPUT_TYPE_BUTTON, type);
            button.value = text;

            this.buttonsBar.appendChild(document.createTextNode(String.fromCharCode(160)));
            this.buttonsBar.appendChild(button);

            var thisObj = this; 
            button.onclick = function() {
                if (typeof handler == Type.FUNCTION) {
                    handler(this);
                } else {
                    thisObj._handleButtonPressed(this);
                }
            }
        },

        getContent: function() {
            return this.contentPane;
        },

        setTitle: function(value) {
            var title = Utils.getTextNode(this.titlePane);
            var newTitle = document.createTextNode(value || document.title);

            if (title != null) {
                this.titlePane.replaceChild(newTitle, title);
            } else {
                this.titlePane.appendChild(newTitle);
            }
        },

        setOpacity: function(value) {
            Utils.setOpacity(this.cover, value || DEFAULT_OPACITY);
        },

        setCssClass: function(value) {
            Utils.addCssClass(this.dialog, value || '');
        },

        setZIndex: function(value) {
            value = parseInt(value);
            Utils.setStyle(this.cover, {zIndex: value});
            Utils.setStyle(this.dialog, {zIndex: value + 1});
            if (this.shim != null) {
                Utils.setStyle(this.shim, {zIndex: value - 1});
            }
        },

        isVisible: function() {
            return Utils.isVisible(this.dialog);
        },

        setVisible: function(visible) {
            var value = visible ? "block" : "none";
            this.dialog.style.display = value;

            if (this.shim != null) {
                this.shim.style.display = value;
            }

            // setups cover visibility
            if (visible) {
                this.cover.style.display = value;
                ModalDialog._VISIBLE_DIALOGS.push(this);
            } else {
                var index = ModalDialog._VISIBLE_DIALOGS.indexOf(this);
                if (index != -1) {
                    ModalDialog._VISIBLE_DIALOGS.splice(index, 1);
                    if (ModalDialog._VISIBLE_DIALOGS.length == 0) {
                        this.cover.style.display = value;
                    }
                }
            }

            if (this.isIENavigator) {
                document.getElementsByTagName("html")[0].style.overflow = visible ? "hidden" : "";
                if (visible) {
                    this._disableTabbedTags();
                } else {
                    this._restoreTabbedTags();
                }
            }

            this._requestFocus();
            this._bringToTop();
        },

        _bringToTop: function() {
            for (var i = 0; i < ModalDialog._VISIBLE_DIALOGS.length; ++i) {
                var dialog = ModalDialog._VISIBLE_DIALOGS[i];
                dialog.setZIndex(dialog == this ? Z_INDEX : (Z_INDEX - 1));
            }
        },

        _createShim: function() {
            if (this.isIENavigator && document.body.insertAdjacentHTML) {
                var html = "<iframe src='about:blank' frameborder='0' scrolling='no' "
                        + "style='position:absolute;display:none;filter:alpha(opacity:0);'></iframe>"
                document.body.insertAdjacentHTML("AfterBegin", html);
                return document.body.firstChild;
            }

            return null;
        },

        _createCover: function() {
            var result = $(COVER_ID);

            if (result == null) {
                result = document.body.appendChild(document.createElement(Tag.DIV));
                result.id = COVER_ID;
                Utils.setStyle(result, this.isIENavigator
                                       ? {position: "absolute", display: "none"}
                                       : {position: "fixed", top: 0, left: 0, width: "100%", height: "100%"});
            }

            return result;
        },

        _compose: function() {
            this.dialog = document.body.appendChild(document.createElement(Tag.DIV));
            this.dialog.id = this.id;
            Utils.addCssClass(this.dialog, MODAL_DIALOG_CSS_CLASS);
            Utils.setStyle(this.dialog, this.isIENavigator
                                        ? {position: "absolute", display: "none"}
                                        : {position: "fixed", top: 0, left: 0});

            this.titlePane = document.createElement(Tag.DIV);
            Utils.addCssClass(this.titlePane, TITLE_CSS_CLASS);
            this.dialog.appendChild(this.titlePane);

            this.contentPane = document.createElement(Tag.DIV);
            Utils.addCssClass(this.contentPane, CONTENT_PANE_CSS_CLASS);
            this.dialog.appendChild(this.contentPane);

            this.buttonsBar = document.createElement(Tag.DIV);
            Utils.addCssClass(this.buttonsBar, BUTTONS_BAR_CSS_CLASS);
            this.dialog.appendChild(this.buttonsBar);
        },

        update: function() {
            //todo: check Safari dialog center position
            var viewportDimension = document.viewport.getDimensions();
            var dialogDimension = Utils.getDimension(this.dialog);
            var dialogTop = Math.max(0, (viewportDimension.height - dialogDimension.height) / 2 - viewportDimension.height * 0.15);
            var dialogLeft = Math.max(0, (viewportDimension.width - dialogDimension.width) / 2);

            if (this.isIENavigator) {
                var viewportScrollOffsets = document.viewport.getScrollOffsets();
                var coverPosition = {top: viewportScrollOffsets.top + 'px', left: viewportScrollOffsets.left + 'px',
                                     width: viewportDimension.width + 'px', height: viewportDimension.height + 'px'};
                if (this.shim != null) {
                    Utils.setStyle(this.shim, coverPosition);
                }

                Utils.setStyle(this.cover, coverPosition);
                Utils.setStyle(this.dialog, {top: viewportScrollOffsets.top + dialogTop + 'px', left: viewportScrollOffsets.left + dialogLeft + 'px'});
            } else {
                Utils.setStyle(this.dialog, {top: dialogTop + 'px', left: dialogLeft + 'px'});
            }
        },

        onSubmit: function() {
        },

        onCancel: function() {
        },

        _requestFocus: function() {
            if (this.isVisible(this.dialog)) {
                var buttons = this.buttonsBar.getElementsByTagName(Tag.INPUT);
                var el = buttons.length > 0 ? buttons[0] : this.getContent();
                try {
                    el.focus();
                } catch (e) {}
            }
        },

        _handleButtonPressed: function(button) {
            if (button.name == ModalDialog.BUTTON_OK) {
                this.onSubmit();
            } else if (button.name == ModalDialog.BUTTON_CANCEL) {
                this.onCancel();
            } else if (button.name == ModalDialog.BUTTON_CLOSE) {
                this.setVisible(false)
            }
        },

        _disableTabbedTags: function() {
            var elements = this._getTabbedElements(document);
            var excludeElements = this._getTabbedElements(this.dialog);
            
            for (var i = 0; i < elements.length; ++i) {
                var el = elements[i];
                if (excludeElements.indexOf(el) == -1) {
                    el.originalDisabled = el.disabled;
                    el.disabled = true;
                }
            }
        },

        _restoreTabbedTags: function() {
            var elements = this._getTabbedElements(document);
            var excludeElements = this._getTabbedElements(this.dialog);

            for (var i = 0; i < elements.length; ++i) {
                var el = elements[i];
                if (excludeElements.indexOf(el) == -1) {
                    if (typeof el.originalDisabled != Type.UNDEFINED) {
                        el.disabled = el.originalDisabled;
                    }
                }
            }
        },

        _getTabbedElements: function(parent) {
            var result = [];
            for (var i = 0; i < TABBED_TAGS.length; ++i) {
                var tags = parent.getElementsByTagName(TABBED_TAGS[i]);
                for (var j = 0; j < tags.length; ++j) {
                    result.push(tags[j]);
                }
            }
            return result;
        }
    }
})();