function ProgressBar(id) {
    this._constructor(id);
}
(function() { // create local scope;

    var PROGRESS_BAR_ID = "progressbar";
    var INFO_CLASS = "pbinfo";
    var BAR_CONTAINER_CLASS = "pbcontainer";
    var PERCENT_CLASS = "pbpercent";
    var BAR_CLASS = "pbbar";

    ProgressBar.prototype = {

        container: null,
        info: null,
        percent: null,
        bar: null,
        lastStatusInfo: null,
        lastTimeInfo: null,

        _constructor: function() {
            this.container = document.getElementById(PROGRESS_BAR_ID);

            var suffix = new Date().getTime();
            this._render(this.container, suffix);
            this.info = document.getElementById(INFO_CLASS + suffix);
            this.percent = document.getElementById(PERCENT_CLASS + suffix);
            this.bar = document.getElementById(BAR_CLASS + suffix);

            this.setVisible(false);
        },

        setProgress: function(progress) {
            progress = Math.max(0, Math.min(100, progress));
            this.bar.style.width = progress + "%";
            this.percent.innerHTML = progress + "%";
        },

        setStatusInfo: function(text) {
            var node = document.createTextNode(text);

            if (this.lastStatusInfo != null) {
                this.info.replaceChild(node, this.lastStatusInfo);
                this.lastStatusInfo = node;
            } else {
                this.lastStatusInfo = this.info.appendChild(node);
            }
        },

        setTimeInfo: function(text) {
            var node = document.createTextNode(text);

            if (this.lastTimeInfo != null) {
                this.info.replaceChild(node, this.lastTimeInfo);
                this.lastTimeInfo = node;
            } else {
                this.info.appendChild(document.createElement("BR"));
                this.lastTimeInfo = this.info.appendChild(node);
            }
        },

        setVisible: function(visible) {
            this.container.style.display = (visible ? "block" : "none");
        },

        _render: function(container, idSuffix) {
            var div = document.createElement("div");
            var content = container.firstChild ? container.insertBefore(div, container.firstChild) : container.parent.appendChild(div);

            content.innerHTML = "<div class='" + INFO_CLASS + "' id='" + INFO_CLASS + idSuffix + "'>&nbsp;</div>"
                                + "<div class='" + BAR_CONTAINER_CLASS + "'>"
                                + "<span class='" + PERCENT_CLASS + "' id='" + PERCENT_CLASS + idSuffix + "'></span>"
                                + "<span class='" + BAR_CLASS + "' id='" + BAR_CLASS + idSuffix + "'></span>"
                                + "</div>";
        }

    }

})();