function UploadPane(form) {
    this._constructor(form);
}
(function() { // create local scope;

    var FILE_CHOOSER_CLASS = "filechooser";
    var FILE_CHOOSER_HIDDEN_CLASS = "hidden";
    var ADD_BUTTON_ID = "addfile";
    var REMOVE_BUTTON_ID = "removefile";
    var UPLOAD_BUTTON_ID = "startupload";
    var FILE_LIST_ID = "filelist";
    var FILE_LIST_IE_MAC_ID = "filelistiemac";
    var FILE_COUNTER_ID = "filecounter";
    var IMAGE_DISABLED_CLASS = "disabled";

    UploadPane.prototype = {

        form: null,
        addButton: null,
        removeButton: null,
        list: null,
        counter: null,
        chooser: null,
        originalChooser: null,
        isIEMacNavigator: null,

        _constructor: function(form) {
            this.form = form;
            this.isIEMacNavigator = Utils.isIEMacNavigator();

            this.addButton = $(ADD_BUTTON_ID);
            this.removeButton = $(REMOVE_BUTTON_ID);
            this.uploadButton = $(UPLOAD_BUTTON_ID);
            this.list = $(FILE_LIST_ID);
            this.counter = $(FILE_COUNTER_ID);
            this.chooser = this._findFileChooser();
            this.originalChooser = this.chooser.cloneNode(true);

            if (this.isIEMacNavigator) {
                Utils.addCssClass(this.chooser, FILE_CHOOSER_CLASS);
                Utils.addCssClass(this.chooser, FILE_CHOOSER_HIDDEN_CLASS);
            } else {
                var thisObj = this;
                this.addButton.onclik = function() { return false; }
                this.removeButton.onclick = function() { thisObj.remove(); return false; };
                this.list.onchange = function() { thisObj.update(); };
                this.chooser.onchange = function() { thisObj.add(this.value); };
                this.chooser.onkeypress = function() { this.blur(); };

                this.update();
            }
        },

        add: function(path) {
            if (this.contains(path)) {
                return;
            }

            var option = this._createOptionItem(path);
            this.list.appendChild(option);

            option.chooser = this.chooser;
            option.chooser.disabled = true;
            Utils.addCssClass(option.chooser, FILE_CHOOSER_CLASS);
            Utils.addCssClass(option.chooser, FILE_CHOOSER_HIDDEN_CLASS);

            var newChooser = this._createFileChooser();
            this.chooser = this.form.appendChild(newChooser);

            this.update();
        },

        remove: function() {
            var selected = -1;

            var i = 0;
            while (this.list.selectedIndex != -1) {
                var option = this.list.options[i];

                if (option.selected) {
                    selected = this.list.selectedIndex;

                    this.form.removeChild(option.chooser);
                    this.list.removeChild(option);
                    option.chooser = null;
                } else {
                    i++;
                }
            }

            if (selected >= this.getItemCount()) {
                selected = this.getItemCount() - 1;
            }

            this._setSelectedIndex(selected);
            this.update();
        },

        contains: function(path) {
            for (var i = 0, size = this.getItemCount(); i < size; ++i) {
                if (this.list.options[i].value == path) {
                    return true;
                }
            }

            return false;
        },

        update: function() {
            var count = this.getItemCount();
            this.counter.innerHTML = count;
            Utils.setEnabled(this.removeButton, this.list.selectedIndex != -1, IMAGE_DISABLED_CLASS);
            Utils.setEnabled(this.uploadButton, count != 0, IMAGE_DISABLED_CLASS);
        },

        getItemCount: function() {
            return this.list.options.length;
        },

        onBeforeUpload: function() {
            this.form.removeChild(this.chooser);

            if (!this.isIEMacNavigator) {
                this.form.removeChild($(FILE_LIST_IE_MAC_ID));

                for (var i = 0; i < this.getItemCount(); ++i) {
                    var option = this.list.options[i];
                    option.chooser.disabled = false;
                }
            }
        },

        _setSelectedIndex: function(index) {
            if (index >= this.getItemCount()) { return; }

            try {
                this.list.selectedIndex = index;
            } catch(e) {
                // fails in IE
                var thisObj = this;
                setTimeout(function() {
                    thisObj.list.selectedIndex = index;
                }, 10);
            }
        },

        _createOptionItem: function(path) {
            var separator = (path.indexOf('/') != -1) ? '/' : '\\';
            var filename = path.substring(path.lastIndexOf(separator) + 1);

            var result = document.createElement(Tag.OPTION);
            result.value = path;
            result.title = path;
            result.appendChild(document.createTextNode(filename));

            return result;
        },

        _createFileChooser: function() {
            var chooser = this.originalChooser.cloneNode(true);
            chooser.name += this.getItemCount();

            var thisObj = this;
            chooser.onchange = function() { thisObj.add(this.value); };
            chooser.onkeypress = function() { this.blur(); };

            return chooser;
        },

        _findFileChooser: function() {
            for (var i = 0; i < this.form.elements.length; ++i) {
                var candidate = this.form.elements[i];

                if (Utils.isFileInput(candidate) && Utils.containsCssClass(candidate, FILE_CHOOSER_CLASS)) {
                    return candidate;
                }
            }

            return null;
        }

    }

})();