var ACTION_FORM_NAME = "actionform";
var RENAME_FORM_NAME = "renameform";
var LIST_TABLE_ID = "list";
var UPLOAD_FORM_ID = "uploadform";
var PARENT_DIR_LINK_ID = "parentdir";
var ROW_SELECTED_CLASS = "sel";
var RENAME_INPUT_CLASS = "rename";
var DIALOG_CHANGE_PWD = "changePasswordDialog";
var DIALOG_EMAIL_FILE = "emailFileDialog";
var NON_BREAK_SPACE_REG = /\u00A0/g;

function getListTable() {
    return document.getElementById(LIST_TABLE_ID);
}

function getUploadForm() {
    return document.getElementById(UPLOAD_FORM_ID);
}

function getActionForm() {
    return document.getElementById(ACTION_FORM_NAME);
}

function navigate(e) {
    var ke = new KeyEvent(e);
    var code = ke.getKeyCode();

    switch (code) {
        case KeyEvent.VK_ENTER:
            if (isVisibleRenameForm()) {
                var params = createParameters(ke.getSource().value);
                new Action(Action.RENAME, getActionForm()).perform(params);
            }
            hideRenameForm();
            break;

        case KeyEvent.VK_ESCAPE:
            hideRenameForm();
            break;
    }
}


function navigateMouse(e) {
    var ke = new KeyEvent(e);
    var elem = ke.getSource();
    var renameForm = getRenameForm(false);

    if (renameForm != null && renameForm.firstChild == elem) {
        renameForm.firstChild.focus();
        ke.cancel();
    } else {
        hideRenameForm();

        var candidate = findParentRow(elem);
        var selectedRow = getSelectedRow();

        if (isRow(candidate)) {
            setRowSelected(selectedRow, false);
            setRowSelected(candidate, true);
        }

        buttonsBarPane.update();
    }
}

function hasMarkedFiles(table) {
    var rows = table.rows;
    for (var i = 0, size = rows.length; i < size; ++i) {
        var checkbox = getFirstCheckbox(rows[i]);
        if (checkbox != null && checkbox.checked) {
            return true;
        }
    }
    return false;
}

function findParentRow(elem) {
    for (var parent = elem.parentNode; parent != null; parent = parent.parentNode) {
        if (isRow(parent)) {
            return parent;
        }
    }

    return null;
}

function setRowSelected(row, selected) {
    if (isRow(row)) {
        row.className = selected ? ROW_SELECTED_CLASS : "";
    }
}

function setDefaultRowSelected() {
    var table = getListTable();

    if (table != null) {
        for (var i = 0; i < table.rows.length; ++i) {
            var candidate = table.rows[i];
            if (isRow(candidate)) {
                setRowSelected(candidate, true);
                break;
            }
        }
    }
}

function showRenameForm(name) {
    var form = getRenameForm(true);
    form.firstChild.value = name;
    form.firstChild.onkeydown = navigate;
    form.firstChild.select();
    form.firstChild.focus();

    getActionForm().oldname.value = name;
}

function hideRenameForm() {
    var form = getRenameForm(false);

    if (form != null && form.parentNode) {
        form.parentNode.removeChild(form);
        getActionForm().oldname.value = "";
    }
}

function getRenameForm(create) {
    var result = document.getElementById(RENAME_FORM_NAME);

    if (result == null && create) {
        var selectedLink = getSelectedLink(true);

        if (selectedLink != null) {
            var control = document.createElement("div");
            control.innerHTML = "<input type='text' />";
            control.id = RENAME_FORM_NAME;
            control.className = RENAME_INPUT_CLASS;
            control.style.width = selectedLink.offsetWidth + "px";
            return selectedLink.parentNode.insertBefore(control, selectedLink);
        }
    }

    return result;
}

function isVisibleRenameForm() {
    return (getRenameForm(false) != null);
}

function markSelectedFile() {
    var row = getSelectedRow();

    if (row != null) {
        var checkbox = getFirstCheckbox(row);

        if (checkbox != null) {
            checkbox.checked = !checkbox.checked;
        }
    }
}

function getMarkedFiles() {
    var result = new Array();
    var rows = getListTable().rows;

    for (var i = 0, size = rows.length; i < size; ++i) {
        var row = rows[i];
        var checkbox = getFirstCheckbox(row);

        if (checkbox != null && checkbox.checked) {
            var filename = getFileName(row);

            if (filename != null) {
                result[result.length] = filename;
            }
        }
    }

    return result;
}

function rowDown() {
    var selectedRow = getSelectedRow();

    if (isRow(selectedRow)) {
        var nextRow = findNextRow(selectedRow);

        if (isRow(nextRow)) {
            setRowSelected(nextRow, true);
            setRowSelected(selectedRow, false);
        }
    } else {
        setDefaultRowSelected();
    }
}

function rowUp() {
    var selectedRow = getSelectedRow();

    if (isRow(selectedRow)) {
        var previousRow = findPreviousRow(selectedRow)

        if (isRow(previousRow)) {
            setRowSelected(previousRow, true);
            setRowSelected(selectedRow, false);
        }
    } else {
        setDefaultRowSelected();
    }
}

function findNextRow(row) {
    return findRowSibling(row, false);
}

function findPreviousRow(row) {
    return findRowSibling(row, true);
}

function findRowSibling(row, previous) {
    var table = getListTable();

    if (table != null && isRow(row)) {
        for (var i = 0; i < table.rows.length; ++i) {
            if (table.rows[i] == row) {
                if (previous && isRow(table.rows[i - 1])) {
                    return table.rows[i - 1];
                } else if (!previous && isRow(table.rows[i + 1])) {
                    return table.rows[i + 1];
                } else {
                    return null;
                }
            }
        }
    }

    return null;
}

function getSelectedRow() {
    var rows = getListTable().rows;

    for (var i = 0, size = rows.length; i < size; ++i) {
        var row = rows[i];

        if (isSelectedRow(row)) {
            return row;
        }
    }

    return null;
}


function isSelectedRow(row) {
    return (isRow(row) && row.className == ROW_SELECTED_CLASS);
}

function isRow(elem) {
    return ((elem != null) && (elem.tagName == "TR") && (elem.cells.length > 0) && (elem.cells[0].tagName == "TD"));
}

function getFirstCheckbox(elem) {
    if (elem != null) {
        var inputs = elem.getElementsByTagName("input");

        return (inputs.length > 0 && Utils.isCheckboxInput(inputs[0])) ? inputs[0] : null;
    }

    return null;
}

function renameAction() {
    var name = getSelectedFile();

    if (name != null) {
        showRenameForm(name);
    } else {
        alert(Messages.NO_FILES_SELECTED);
    }
}

function switchToAppletUI() {
    new Action(Action.UI_APPLET, getActionForm()).perform();
}

function refreshAction() {
    new Action(Action.REFRESH, getActionForm()).perform(createParameters());
}

function changeToParentDirectoryAction() {
    new Action(Action.CDUP, getActionForm()).perform(createParameters());
}

function changeWorkingDirectoryAction(path) {
    new Action(Action.CWD, getActionForm()).perform(createParameters(path));
}

function cancelUploadAction() {
    if (uploader == null) {
        return;
    }

    uploader.cancel();
    uploader = null;
    
    new Action(Action.UPLOAD_CANCEL, getActionForm()).perform();
}

function newDirectoryAction() {
    new NewDirectoryDialog().setVisible(true);
}

function deleteAction() {
    hideRenameForm();
    var selected = getSelectedFile();
    var marked = getMarkedFiles();

    if (selected == null && marked.length == 0) {
        alert(Messages.NO_FILES_SELECTED);
        return;
    }

    if (marked.length == 0) {
        markSelectedFile();

        if (confirm(Messages.CONFIRM_DELETE)) {
            new Action(Action.DELETE, getActionForm()).perform(createParameters(selected));
        } else {
            // this unmark selected file
            markSelectedFile();
        }
    } else if (confirm(Messages.CONFIRM_DELETE)) {
        new Action(Action.DELETE, getActionForm()).perform(createParameters(marked));
    }
}

function sendEmailAction() {
    hideRenameForm();
    var selected = getSelectedFile();
    var marked = getMarkedFiles();
    var params = {};

    if (marked.length != 0) {
        params = createParameters(marked);
    } else if (selected != null) {
        params = createParameters(selected);
    }

    showDialog("", DIALOG_EMAIL_FILE, 550, 495);
    new Action(Action.EMAIL, getActionForm()).perform(params, DIALOG_EMAIL_FILE);
}

function createParameters(filenames) {
    var sortedColumnIndex = (window.sorter) ? sorter.getSortedColumnIndex() : null;
    var sortedAscending = (window.sorter) ? sorter.isSortedAscending() : null;

    if (arguments.length == 1) {
        return {
            filename: filenames,
            sortedColumnIndex: sortedColumnIndex,
            sortedAscending: sortedAscending};
    } else {
        return {
            sortedColumnIndex: sortedColumnIndex,
            sortedAscending: sortedAscending};
    }
}

function getFileName(elem) {
    if (elem != null) {
        var links = elem.getElementsByTagName("a");
        return (links.length > 0) ? Utils.getInnerText(links[0]) : null;
    }

    return null;
}

function getSelectedFile() {
    var link = getSelectedLink(true);
    return link != null ? Utils.getInnerText(link).replace(NON_BREAK_SPACE_REG, " ") : null;
}

function getSelectedLink(excludeParentDir) {
    var selected = getSelectedRow();

    if (isRow(selected)) {
        var links = selected.getElementsByTagName("a");

        if (links.length == 1) {
            if (excludeParentDir) {
                return (links[0].id != PARENT_DIR_LINK_ID) ? links[0] : null;
            } else {
                return links[0];
            }
        }
    }

    return null;
}

//todo: move to table
function selectAll(table, value) {
    var rows = table.rows;

    for (var i = 0, size = rows.length; i < size; ++i) {
        var checkbox = getFirstCheckbox(rows[i]);

        if (checkbox != null) {
            checkbox.checked = value;
        }
    }
    buttonsBarPane.update();
}

function showDialog(url, id, width, height) {
    var left = (screen.width - width) / 2;
    var top = (screen.height - height) / 2;
    stopInactivityService();
    var w = window.open(url, id, "scrollbars,status=no,menubar=no,width=" + width + ",height=" + height + ",top=" + top + ",left=" + left);
    w.focus();
    startInactivityService();

    return w;
}

function onStartUpload(form) {
    var selected = false;

    for (var i = 0; i < form.elements.length; ++i) {
        var elem = form.elements[i];

        if (Utils.isFileInput(elem) && (elem.value != "")) {
            selected = true;
            break;
        }
    }

    if (selected) {
        uploadPane.onBeforeUpload();
        stopInactivityService();
        uploader = new UploadService(UPLOAD_PREPARE_PROGRESS_URL);
        uploader.setTimeout(CONNECTION_TIMEOUT);
        uploader.start(form);
    } else {
        alert(Messages.NO_FILES_SELECTED);
    }

    return false;
}

function onPrepareUpload(percent, message, remaining) {
    if (uploader != null) {
        uploader.prepare(percent, message, remaining);
    }
}

function onPrepareComplete() {
    if (uploader != null) {
        uploader.prepareComplete();
    }
}

function onProgressUpload(percent, message, timeMessage) {
    if (uploader != null) {
        uploader.progress(percent, message, timeMessage);
    }
}

function onFinishUpload() {
    if (uploader != null) {
        uploader.finish();
        uploader = null;
        refreshAction();
    }
}

function onErrorUpload(message) {
    onFinishUpload();
    alert(message);
}

function onConnectionTimeout() {
    alert(Messages.CONNECTION_TIMEOUT);
    refreshAction();
}

function onSuccess(message) {
    alert(message);
    restartInactivityService();
}

function startInactivityService() {
    if (window.inactivityService) {
        inactivityService.start();
    }
}

function stopInactivityService() {
    if (window.inactivityService) {
        inactivityService.stop();
    }
}

function restartInactivityService() {
    if (window.inactivityService) {
        inactivityService.restart();
    }
}